<?php
/**
*  @author    AGENCE WEB 360 <contact@agence-web-360.com>
*  @copyright 2015-2017 AGENCE WEB 360
*  Property of AGENCE WEB 360
*/
require_once 'UPSBaseApi.php';
require_once 'UPSMultiPackageApi.php';

class UPSDGShippingApi extends UPSBaseApi
{
  /**
  * Send a ship request, and returns the tracking number and the base64 encoded GIF label.
  * @param string $shipper_name
  * @param string $shipper_number
  * @param string $shipper_attention_name
  * @param string $shipper_tax_id_number
  * @param array $shipper_address address formated by UPSBaseApi::formatAddress
  * @param array $shipper_phone_number
  * @param string $shipto_name
  * @param string $shipto_attention_name
  * @param array $shipto_address address formated by UPSBaseApi::formatAddress
  * @param string $shipto_phone_number
  * @param float $weight
  * @param string $serviceCode
  * @param string $description
  * @param bool $negotiatedPrices if true response will return account based negociated prices
  * @param array $accessorials formated by UPSBaseApi::formatAccessorials
  * @param null $accesspoint_name
  * @param null $accesspoint_id
  * @param null $accesspoint_address formated by UPSBaseApi::formatAddress
  * @param string $ShipmentIndicationType
  * @param null|array $dimensions
  * @param string $weightUnitCode KGS (default) or LBS
  * @param string $dimensionUnitCode CM (default) or IN
  * @param bool $LargePackageIndicator false (default), true if large package
  * @param null $shipfrom_name if null $shipper_name is used
  * @param null $shipfrom_attention_name if null $shipper_attention_name is used
  * @param null $shipfrom_address if null $shipper_address is used, else address formated by
  *          UPSBaseApi::formatAddress
  * @param null $shipfrom_phone_number if null $shipper_phone_number is used
  * @param string $PickupType
  * @param string $CustomerClassification
  * @return array|bool false if no response exists, else array('tracking_number' (string), 'gif_base64' (xstring))
  * @throws Exception
  */
  public function doShipment(
    $shipper_name,
    $shipper_number,
    $shipper_attention_name,
    $shipper_tax_id_number,
    $shipper_address,
    $shipper_phone_number,
    $shipto_name,
    $shipto_attention_name,
    $shipto_address,
    $shipto_phone_number,
    $weight,
    $serviceCode = '11',
    $description = '',
    $negotiatedPrices = false,
    $accessorials = array(),
    $accesspoint_name = null,
    $accesspoint_id = null,
    $accesspoint_address = null,
    $ShipmentIndicationType = '02',
    $dimensions = null,
    $weightUnitCode = 'KGS',
    $dimensionUnitCode = 'CM',
    $LargePackageIndicator = false,
    $shipfrom_name = null,
    $shipfrom_attention_name = null,
    $shipfrom_address = null,
    $shipfrom_phone_number = null,
    $PickupType = '01',
    $CustomerClassification = '01',
    $notificationEmail = '',
    $notificationLang = 'en',
    $additionalHandling = false,
    $formatlabel = 'GIF',
    $nbrColis = 1,
    $factureNum=00,
    $countryToCode=false,
    $orderID = null
  ) {
    $UPSMultiPackage = new UPSMultiPackageApi(
      Configuration::get("PS_MO_UPS_ACCESSKEY"),
      Configuration::get("PS_MO_UPS_USERID"),
      Configuration::get("PS_MO_UPS_PASSWD"),
      ((int)Configuration::get("PS_MO_UPS_MODE")) == 0,
      Configuration::get("PS_MO_UPS_LOGS")
    );
    if ($shipfrom_name === null) {
      $shipfrom_name = $shipper_name;
    }
    if ($shipfrom_address === null) {
      $shipfrom_address = array_merge($shipper_address);
    }
    if ($shipfrom_attention_name === null) {
      $shipfrom_attention_name = $shipper_attention_name;
    }
    if ($shipfrom_phone_number === null) {
      $shipfrom_phone_number = $shipper_phone_number;
    }

    $shipto_phone_number = str_replace(array(" ",".","/","-"), "", $shipto_phone_number);

    // Gestion multi-colis
    if ($nbrColis > 1) {
      $weight = ceil(floatval($weight)/$nbrColis);
    }

    if ($weightUnitCode == "LBS") {
      $weight = ceil(floatval($weight));
      if ($weight<=0) {
        $weight = 1;
      }
    } else {
      $weight = ceil(floatval($weight)*2)/2;
      if ($weight<=0) {
        $weight = 0.5;
      }
    }

    $this->setEndPointURL('webservices/Ship');
    $this->setWSDL(dirname(__FILE__)."/Shipping/Ship.wsdl");
    $this->setOperation("ProcessShipment");
    $regulationSet = ($serviceCode == '11') ? 'ADR' : 'IATA';
    $request = array(
      'Request' => array(
        'RequestOption' => 'nonvalidate'
      ),
      'Shipment' => array(
        'Description' => $description,
        'ReferenceNumber' =>  array(
          'Value' => $factureNum,
        ),
        'PickupType' => array(
          'Code' => $PickupType,
        ),
        'CustomerClassification' => array(
          'Code' => $CustomerClassification,
        ),
        'Shipper' => array(
          'Name' => $shipper_name,
          'AttentionName' => $shipper_attention_name,
          'TaxIdentificationNumber'=> $shipper_tax_id_number,
          'ShipperNumber' => $shipper_number,
          'Address' => $shipper_address,
          'Phone' => array(
            'Number' => $shipper_phone_number,
          )
        ),
        'ShipTo' => array(
          'Name' => $shipto_name,
          'AttentionName' => $shipto_attention_name,
          'Address' => $shipto_address,
          'Phone' => array(
            'Number' => $shipto_phone_number,
          )
        ),
        'ShipFrom' => array(
          'Name' => $shipfrom_name,
          'AttentionName' => $shipfrom_attention_name,
          'Address' => $shipfrom_address,
          'Phone' => array(
            'Number' => $shipfrom_phone_number,
          )
        ),
        'Service' => array(
          'Code' => $serviceCode,
        ),
        'PaymentInformation' => array(
          'ShipmentCharge' => array(
            'Type' => '01',
            'BillShipper' => array(
              'AccountNumber' => $shipper_number
            )
          )
        ),
        'Package' => array(
          array(
            'Packaging' => array(
              'Code' => '02',//always 02 as stated in the requirements
            ),
            'PackageWeight' => array(
              'Weight' => $weight,
              'UnitOfMeasurement' => array(
                'Code' => $weightUnitCode,
              )
            ),
            'PackageServiceOptions' => array(
              'HazMat' => array(
                'PackagingTypeQuantity' => '1',
                'ClassDivisionNumber' => '3',
                'Quantity' => '12',
                'UOM' => 'ml',
                'PackagingType' => 'Fiberboard Box',
                'IDNumber' => '1266',
                'AdditionalDescription' => 'Limited quantity',
                'PackagingGroupType' => 'II',
                'ProperShippingName' => 'Perfumery products',
                'RegulationSet' => $regulationSet,
                'TransportationMode' => 'Ground',
                'CommodityRegulatedLevelCode' => 'LQ',
                'TransportCategory' => '2',
                'TunnelRestrictionCode' => 'D/E',
                'ChemicalRecordIdentifier' => '1266',
                'PackagingInstructionCode'=>'1266'
              ),
              'PackageIdentifier' => rand(),
            ),
          )
        ),
        'ShipmentServiceOptions' => array(),

        'InternationalForms' => array(
          'CN22Form' => array(
            'LabelPrintType' => $formatlabel
          )
        )
      ),'LabelSpecification' => array(
        'LabelImageFormat' => array(
          'Code' => $formatlabel
        ),
      ),
    );

    if ($formatlabel === "ZPL") {
      $request['LabelSpecification']['LabelStockSize'] = array();
      $request['LabelSpecification']['LabelStockSize']['Height'] = 6;
      $request['LabelSpecification']['LabelStockSize']['Width'] = 4;
    }

    if ($negotiatedPrices) {
      $request['Shipment']['ShipmentRatingOptions'] = array();
      $request['Shipment']['ShipmentRatingOptions']['NegotiatedRatesIndicator'] = '';
    }

    if (isset($accessorials["DeclaredValue"]) && is_array($accessorials["DeclaredValue"])) {
      $request['Shipment']['Package'][0]['PackageServiceOptions']['DeclaredValue'] =
      $accessorials["DeclaredValue"];
    }

    if (isset($accessorials["Signature"]) && $accessorials["Signature"] !== false) {
      $request['Shipment']['ShipmentServiceOptions']['DeliveryConfirmation'] =
      array('DCISType' => $accessorials["Signature"]);
    }

    if (isset($accessorials["CarbonNeutral"]) && $accessorials["CarbonNeutral"]) {
      $request['Shipment']['ShipmentServiceOptions']['UPScarbonneutralIndicator'] = '';
    }

    if ($accesspoint_address !== null && $accesspoint_name !== null) {
      $ShipmentIndicationType = '01';
      if (isset($accessorials["AccessPointCOD"]) && is_array($accessorials["AccessPointCOD"])) {
        $request['Shipment']['ShipmentServiceOptions']['AccessPointCOD'] = $accessorials["AccessPointCOD"];
      }
      if (isset($accessorials['ToAddresseeOnly']) && $accessorials['ToAddresseeOnly']) {
        $request['Shipment']['ShipmentServiceOptions']['DeliverToAddresseeOnlyIndicator'] = '';
      }
      $request['Shipment']['ShipmentIndicationType'] = array(
        'Code' => $ShipmentIndicationType
      );
      $l = '';
      switch ($notificationLang) {
        case 'fr':
        $l = 'FRA';
        break;
        case 'nl':
        $l = 'NLD';
        break;
        case 'es':
        $l = "SPA";
        break;
        default:
        $l = "ENG";
        break;
      }

      $request['Shipment']['ShipmentServiceOptions'] = array(
        'Notification' =>
        array(
          'NotificationCode' => '012',
          //'NotificationCode' => '013',
          'EMail' => array(
            'EMailAddress' => $notificationEmail
          ),

          'Locale' => array(
            //'Language' => "ENG",
            //'Dialect' => 'US'
            'Language' => $l,
            'Dialect' => ($l == 'ENG' ? 'US' : '97')
          ),

        ),
        array(
          'NotificationCode' => '7',
          //'NotificationCode' => '013',
          'EMail' => array(
            'EMailAddress' => array($notificationEmail,Configuration::get('PS_MO_UPS_NOTIFICATION_SEND_ADMIN_EXCEPTION') ?  : Configuration::get('PS_SHOP_EMAIL'))
          ),

          'Locale' => array(
            //'Language' => "ENG",
            //'Dialect' => 'US'
            'Language' => $l,
            'Dialect' => ($l == 'ENG' ? 'US' : '97')
          )
        )
      );

      if(strtoupper($countryToCode!='US')){
        $request['Shipment']['ShipmentServiceOptions']['Notification']['TextMessage']=array(
          'PhoneNumber' => $shipto_phone_number
          //'PhoneNumber' => '473-1000'
        );

      }
      $request['Shipment']['AlternateDeliveryAddress'] = array(
        'Name' => $accesspoint_name,
        'AttentionName' => $accesspoint_name,
        'UPSAccessPointID' => $accesspoint_id,
        'Address' => $accesspoint_address
      );
    }

    if (is_array($dimensions) && count($dimensions) == 3) {
      $request['Shipment']['Package'][0]['Dimensions'] = array(
        'Length' => $dimensions[0],
        'Width' => $dimensions[1],
        'Height' => $dimensions[2],
        'UnitOfMeasurement' => array(
          'Code' => $dimensionUnitCode,
        )
      );
    }

    if ($LargePackageIndicator) {
      $request['Shipment']['Package'][0]['LargePackageIndicator'] = '';
    }

    if ($additionalHandling) {
      $request['Shipment']['Package'][0]['AdditionalHandlingIndicator'] = '';
    }
    

    for ($iNbrColisTmp=1; $iNbrColisTmp < $nbrColis; $iNbrColisTmp++) {
      
      $request['Shipment']['Package'][$iNbrColisTmp] = $request['Shipment']['Package'][0];
      $request['Shipment']['Package'][$iNbrColisTmp]['Packaging']['Code'] = '02';
      $request['Shipment']['Package'][$iNbrColisTmp]['PackageWeight']['Weight'] = $weight;
      $request['Shipment']['Package'][$iNbrColisTmp]['PackageWeight']['UnitOfMeasurement']['Code'] = $weightUnitCode;
      $request['Shipment']['Package'][$iNbrColisTmp]['PackageServiceOptions'] = array(
        'HazMat' => array(
          'PackagingTypeQuantity' => '1',
          'ClassDivisionNumber' => '3',
          'Quantity' => '12',
          'UOM' => 'ml',
          'PackagingType' => 'Fiberboard Box',
          'IDNumber' => '1266',
          'AdditionalDescription' => 'Limited quantity',
          'PackagingGroupType' => 'II',
          'ProperShippingName' => 'Perfumery products',
          'RegulationSet' => $regulationSet,
          'TransportationMode' => 'Ground',
          'CommodityRegulatedLevelCode' => 'LQ',
          'TransportCategory' => '2',
          'TunnelRestrictionCode' => 'D/E',
          'ChemicalRecordIdentifier' => '1266',
          'PackagingInstructionCode'=>'1266'
        ),
        'PackageIdentifier' => rand(),
      );
     
    }

    $request['Shipment']['Package'][0]['PackageWeight']['UnitOfMeasurement']['Code'] = $weightUnitCode;

    if(Configuration::get('PS_MO_UPS_MULTI_COLIS', 0) == 1 && sizeof($dimensions) < 3){
      $order=new Order($orderID);
      $packagesFormatted = $UPSMultiPackage->doPackage($order->getProducts(), $order->getTotalWeight(), $weightUnitCode, $dimensionUnitCode);
      if($packagesFormatted !== false)
      $request['Shipment']['Package'] = $packagesFormatted;
    }     
    $request['Shipment']['DGSignatoryInfo'] = array(
      'Name' => 'packname',
      'Title' => 'packtitle',
      'Place' => 'Paris',
      'ShipperDeclaration' => '02'
    );
      $request['Shipment']['Description']=$description;
      $request['Shipment']['ReferenceNumber']=array(
        'Value' => rand(),
      );
    //  print_r($request);exit;
    $this->setRequest($request);
    $this->send();
    $response = $this->getResponse();
    
    if ($response === false) {
      return false;
    }

    if ($response->Response->ResponseStatus->Code != '1') {
      throw new Exception("UPSShippingApi - doShipment : an error occured " .
      $response->Response->ResponseStatus->Code . " - " . $response->Response->ResponseStatus->Description);
    }

    if (!is_array($response->ShipmentResults->PackageResults)){
      $tracking_number = array($response->ShipmentResults->PackageResults->TrackingNumber);
      $gif_base64 = array($response->ShipmentResults->PackageResults->ShippingLabel->GraphicImage);
    } else {
      $tracking_number = array();
      $gif_base64 = array();
      foreach ($response->ShipmentResults->PackageResults as $PackageResultsTmp) {
        $tracking_number[] = $PackageResultsTmp->TrackingNumber;
        $gif_base64[] = $PackageResultsTmp->ShippingLabel->GraphicImage;
      }
    }
    if (isset($response->ShipmentResults->DGPaperImage)){
      $pdf_dgDocs = $response->ShipmentResults->DGPaperImage;
    }

    return array('tracking_number' => $tracking_number, 'gif_base64' => $gif_base64,'pdf_dgDocs'=>$pdf_dgDocs);
  }

  /**
  * Send a ship request, and returns the tracking number and the base64 encoded GIF label.
  * @param string $shipper_name
  * @param string $shipper_number
  * @param string $shipper_attention_name
  * @param string $shipper_tax_id_number
  * @param array $shipper_address address formated by UPSBaseApi::formatAddress
  * @param array $shipper_phone_number
  * @param string $shipto_name
  * @param string $shipto_attention_name
  * @param array $shipto_address address formated by UPSBaseApi::formatAddress
  * @param string $shipto_phone_number
  * @param float $weight
  * @param string $serviceCode
  * @param string $description
  * @param bool $negotiatedPrices if true response will return account based negociated prices
  * @param array $accessorials formated by UPSBaseApi::formatAccessorials
  * @param null $accesspoint_name
  * @param null $accesspoint_id
  * @param null $accesspoint_address formated by UPSBaseApi::formatAddress
  * @param string $ShipmentIndicationType
  * @param null|array $dimensions
  * @param string $weightUnitCode KGS (default) or LBS
  * @param string $dimensionUnitCode CM (default) or IN
  * @param bool $LargePackageIndicator false (default), true if large package
  * @param null $shipfrom_name if null $shipper_name is used
  * @param null $shipfrom_attention_name if null $shipper_attention_name is used
  * @param null $shipfrom_address if null $shipper_address is used, else address formated by
  *          UPSBaseApi::formatAddress
  * @param null $shipfrom_phone_number if null $shipper_phone_number is used
  * @param string $PickupType
  * @param string $CustomerClassification
  * @return array|bool false if no response exists, else array('tracking_number' (string), 'gif_base64' (xstring))
  * @throws Exception
  */
  public function doShipmentRetour(
    $shipper_name,
    $shipper_number,
    $shipper_attention_name,
    $shipper_tax_id_number,
    $shipper_address,
    $shipper_phone_number,
    $shipto_name,
    $shipto_attention_name,
    $shipto_address,
    $shipto_phone_number,
    $weight,
    $serviceCode = '11',
    $description = '',
    $negotiatedPrices = false,
    $accessorials = array(),
    $accesspoint_name = null,
    $accesspoint_id = null,
    $accesspoint_address = null,
    $ShipmentIndicationType = '02',
    $dimensions = null,
    $weightUnitCode = 'KGS',
    $dimensionUnitCode = 'CM',
    $LargePackageIndicator = false,
    $shipfrom_name = null,
    $shipfrom_attention_name = null,
    $shipfrom_address = null,
    $shipfrom_phone_number = null,
    $PickupType = '01',
    $CustomerClassification = '01',
    $notificationEmail = '',
    $notificationLang = 'en',
    $additionalHandling = false,
    $formatlabel = 'GIF',
    $nbrColis = 1,
    $orderID = null
  ) {
    $UPSMultiPackage = new UPSMultiPackageApi(
      Configuration::get("PS_MO_UPS_ACCESSKEY"),
      Configuration::get("PS_MO_UPS_USERID"),
      Configuration::get("PS_MO_UPS_PASSWD"),
      ((int)Configuration::get("PS_MO_UPS_MODE")) == 0,
      Configuration::get("PS_MO_UPS_LOGS")
    );
    if ($shipfrom_name === null) {
      $shipfrom_name = $shipper_name;
    }
    if ($shipfrom_address === null) {
      $shipfrom_address = array_merge($shipper_address);
    }
    if ($shipfrom_attention_name === null) {
      $shipfrom_attention_name = $shipper_attention_name;
    }
    if ($shipfrom_phone_number === null) {
      $shipfrom_phone_number = $shipper_phone_number;
    }

    $shipto_phone_number = str_replace(array(" ",".","/","-"), "", $shipto_phone_number);

    // Gestion multi-colis
    if ($nbrColis > 1) {
      $weight = ceil(floatval($weight)/$nbrColis);
    }

    if ($weightUnitCode == "LBS") {
      $weight = ceil(floatval($weight));
      if ($weight<=0) {
        $weight = 1;
      }
    } else {
      $weight = ceil(floatval($weight)*2)/2;
      if ($weight<=0) {
        $weight = 0.5;
      }
    }

    $this->setEndPointURL('webservices/Ship');
    $this->setWSDL(dirname(__FILE__)."/Shipping/Ship.wsdl");
    $this->setOperation("ProcessShipment");

    $request = array(
      'Request' => array(
        'RequestOption' => 'nonvalidate'
      ),
      'Shipment' => array(
        'Description' => $description,
        'PickupType' => array(
          'Code' => $PickupType,
        ),
        'CustomerClassification' => array(
          'Code' => $CustomerClassification,
        ),
        'Shipper' => array(
          'Name' =>  $shipper_name,
          'AttentionName' =>$shipper_attention_name ,
          'TaxIdentificationNumber'=> $shipper_tax_id_number,
          'ShipperNumber' => $shipper_number,
          'Address' =>$shipper_address ,
          'Phone' => array(
            'Number' => $shipper_phone_number ,
          )
        ),

        'ShipTo' => array(
          'Name' =>$shipfrom_name,
          'AttentionName' => $shipfrom_attention_name,
          'Address' => $shipfrom_address,
          'Phone' => array(
            'Number' => $shipfrom_phone_number//$shipper_phone_number,
          )
        ),
        'ShipFrom' => array(
          'Name' => $shipto_name,
          'AttentionName' =>$shipto_attention_name ,
          'Address' => $shipto_address,
          'Phone' => array(
            'Number' => $shipper_phone_number,
          )
        ),
        'Service' => array(
          'Code' => $serviceCode,
        ),
        'PaymentInformation' => array(
          'ShipmentCharge' => array(
            'Type' => '01',
            'BillShipper' => array(
              'AccountNumber' => $shipper_number
            )
          )
        ),
        'Package' => array(
          array(
            'Description' => $description,
            'Packaging' => array(
              'Code' => '02',//always 02 as stated in the requirements
            ),
            'PackageWeight' => array(
              'Weight' => $weight,
              'UnitOfMeasurement' => array(
                'Code' => $weightUnitCode,
              )
            ),
            'PackageServiceOptions' => array()
          )
        ),
        'ShipmentServiceOptions' => array(),

        'InternationalForms' => array(
          'CN22Form' => array(
            'LabelPrintType' => $formatlabel
          )
        )
      ),'LabelSpecification' => array(
        'LabelImageFormat' => array(
          'Code' => $formatlabel
        ),
      ),
    );

    $request['Shipment']['ReturnService']['Code'] = '9'; // Gestion des étiquettes retour : Code PRL

    if ($formatlabel === "ZPL") {
      $request['LabelSpecification']['LabelStockSize'] = array();
      $request['LabelSpecification']['LabelStockSize']['Height'] = 6;
      $request['LabelSpecification']['LabelStockSize']['Width'] = 4;
    }

    if ($negotiatedPrices) {
      $request['Shipment']['ShipmentRatingOptions'] = array();
      $request['Shipment']['ShipmentRatingOptions']['NegotiatedRatesIndicator'] = '';
    }

    if (isset($accessorials["DeclaredValue"]) && is_array($accessorials["DeclaredValue"])) {
      $request['Shipment']['Package'][0]['PackageServiceOptions']['DeclaredValue'] =
      $accessorials["DeclaredValue"];
    }

    if (isset($accessorials["Signature"]) && $accessorials["Signature"] !== false) {
      $request['Shipment']['ShipmentServiceOptions']['DeliveryConfirmation'] =
      array('DCISType' => $accessorials["Signature"]);
    }

    if (isset($accessorials["CarbonNeutral"]) && $accessorials["CarbonNeutral"]) {
      $request['Shipment']['ShipmentServiceOptions']['UPScarbonneutralIndicator'] = '';
    }

    if ($accesspoint_address !== null && $accesspoint_name !== null) {
      $ShipmentIndicationType = '01';
      if (isset($accessorials["AccessPointCOD"]) && is_array($accessorials["AccessPointCOD"])) {
        $request['Shipment']['ShipmentServiceOptions']['AccessPointCOD'] = $accessorials["AccessPointCOD"];
      }
      if (isset($accessorials['ToAddresseeOnly']) && $accessorials['ToAddresseeOnly']) {
        $request['Shipment']['ShipmentServiceOptions']['DeliverToAddresseeOnlyIndicator'] = '';
      }
      $request['Shipment']['ShipmentIndicationType'] = array(
        'Code' => $ShipmentIndicationType
      );
      $l = '';
      switch ($notificationLang) {
        case 'fr':
        $l = 'FRA';
        break;
        case 'nl':
        $l = 'NLD';
        break;
        case 'es':
        $l = "SPA";
        break;
        default:
        $l = "ENG";
        break;
      }
      $request['Shipment']['ShipmentServiceOptions'] = array(
        'Notification' => array(
          'NotificationCode' => '8',
          'EMail' => array(
            'EMailAddress' => $notificationEmail
          ),
          'TextMessage' => array(
            'PhoneNumber' => $shipto_phone_number
          ),
          'Locale' => array(
            'Language' => $l,
            'Dialect' => ($l == 'ENG'?'US':'97')
          )
        )
      );
      $request['Shipment']['AlternateDeliveryAddress'] = array(
        'Name' => $accesspoint_name,
        'AttentionName' => $accesspoint_name,
        'UPSAccessPointID' => $accesspoint_id,
        'Address' => $accesspoint_address
      );
    }

    if (is_array($dimensions) && count($dimensions) == 3) {
      $request['Shipment']['Package'][0]['Dimensions'] = array(
        'Length' => $dimensions[0],
        'Width' => $dimensions[1],
        'Height' => $dimensions[2],
        'UnitOfMeasurement' => array(
          'Code' => $dimensionUnitCode,
        )
      );
    }

    if ($LargePackageIndicator) {
      $request['Shipment']['Package'][0]['LargePackageIndicator'] = '';
    }

    if ($additionalHandling) {
      $request['Shipment']['Package'][0]['AdditionalHandlingIndicator'] = '';
    }

    for ($iNbrColisTmp=1; $iNbrColisTmp < $nbrColis; $iNbrColisTmp++) {
      $request['Shipment']['Package'][$iNbrColisTmp] = $request['Shipment']['Package'][0];
      $request['Shipment']['Package'][$iNbrColisTmp]['Packaging']['Code'] = '02';
      $request['Shipment']['Package'][$iNbrColisTmp]['PackageWeight']['Weight'] = $weight;
      $request['Shipment']['Package'][$iNbrColisTmp]['PackageWeight']['UnitOfMeasurement']['Code'] = $weightUnitCode;
    }
    $request['Shipment']['Package'][0]['PackageWeight']['UnitOfMeasurement']['Code'] = $weightUnitCode;

    if(Configuration::get('PS_MO_UPS_MULTI_COLIS', 0) == 1 && sizeof($dimensions) < 3){
      $order=new Order($orderID);
      $packagesFormatted = $UPSMultiPackage->doPackage($order->getProducts(), $order->getTotalWeight(), $weightUnitCode, $dimensionUnitCode);
      if($packagesFormatted !== false)
      $request['Shipment']['Package'] = $packagesFormatted;
      $request['Shipment']['Package']['Description'] = $description;
      $request['Shipment']['Package']['Packaging']['Code'] = '02';
      $request['Shipment']['Package']['PackageWeight']['Weight'] = $weight;
      $request['Shipment']['Package']['PackageWeight']['UnitOfMeasurement']['Code'] = $weightUnitCode;
    }

    $this->setRequest($request);
    $this->send();
    $response = $this->getResponse();
    //print_r($response);
    if ($response === false) {
      return false;
    }


    if ($response->Response->ResponseStatus->Code != '1') {
      throw new Exception("UPSShippingApi - doShipment : an error occured " .
      $response->Response->ResponseStatus->Code . " - " . $response->Response->ResponseStatus->Description);
    }

    if ($nbrColis == 1) {
      $tracking_number = array($response->ShipmentResults->PackageResults->TrackingNumber);
      $gif_base64 = array($response->ShipmentResults->PackageResults->ShippingLabel->GraphicImage);
    } else {
      $tracking_number = array();
      $gif_base64 = array();
      foreach ($response->ShipmentResults->PackageResults as $PackageResultsTmp) {
        $tracking_number[] = $PackageResultsTmp->TrackingNumber;
        $gif_base64[] = $PackageResultsTmp->ShippingLabel->GraphicImage;
      }
    }

    return array('tracking_number' => $tracking_number, 'gif_base64' => $gif_base64);
  }
}
